@@ -1,21 +1,22 @@
+import time
 import copy
 import random
-import time
-
-import torch
-from torch.utils.data import DataLoader
-import numpy as np
 import operator
 
+import torch
+import numpy as np
+from torch.utils.data import DataLoader
+from torch.cuda.amp import GradScaler
+
+from lightwood.logger import log
+from lightwood.mixers import BaseMixer
+from lightwood.config.config import CONFIG
 from lightwood.helpers.torch import LightwoodAutocast
+from lightwood.constants.lightwood import COLUMN_DATA_TYPES
 from lightwood.mixers.helpers.default_net import DefaultNet
 from lightwood.mixers.helpers.selfaware import SelfAware
 from lightwood.mixers.helpers.ranger import Ranger
 from lightwood.mixers.helpers.transform_corss_entropy_loss import TransformCrossEntropyLoss
-from lightwood.config.config import CONFIG
-from lightwood.constants.lightwood import COLUMN_DATA_TYPES
-from lightwood.mixers import BaseMixer
-from lightwood.logger import log
 
 
 class NnMixer(BaseMixer):
@@ -547,6 +548,9 @@ class NnMixer(BaseMixer):
                 sampler=self._nonpersistent['sampler']
             )
 
+        scaler = GradScaler()
+        selfaware_scaler = GradScaler()
+
         for epoch in range(max_epochs):  # loop over the dataset multiple times
             running_loss = 0.0
             error = 0
@@ -600,10 +604,10 @@ class NnMixer(BaseMixer):
                         target_loss = target_loss.tolist()
                         if type(target_loss[0]) == type([]):
                             target_loss = [np.mean(x) for x in target_loss]
-                        for i, value in enumerate(target_loss):
-                            if len(unreduced_losses) <= i:
+                        for j, value in enumerate(target_loss):
+                            if len(unreduced_losses) <= j:
                                 unreduced_losses.append([])
-                            unreduced_losses[i].append(value)
+                            unreduced_losses[j].append(value)
 
                     unreduced_losses = torch.Tensor(unreduced_losses).to(self.net.device)
 
@@ -615,11 +619,24 @@ class NnMixer(BaseMixer):
                 if CONFIG.MONITORING['batch_loss']:
                     self.monitor.plot_loss(loss.item(), self.total_iterations, 'Targets Batch Loss')
 
-                if awareness_loss is not None:
-                    awareness_loss.backward(retain_graph=True)
-
                 running_loss += loss.item()
-                loss.backward()
+
+                if LightwoodAutocast.active:
+                    scaler.scale(loss).backward()
+                    scaler.step(self.optimizer)
+                    scaler.update()
+                    if self.is_selfaware and self.start_selfaware_training and awareness_loss is not None:
+                        selfaware_scaler.scale(awareness_loss).backward()
+                        selfaware_scaler.step(self.selfaware_optimizer)
+                        selfaware_scaler.update()
+                else:
+                    loss.backward()
+                    self.optimizer.step()
+                    if self.is_selfaware and self.start_selfaware_training and awareness_loss is not None:
+                        awareness_loss.backward(retain_graph=True)
+                        self.selfaware_optimizer.step()
+
+                error = running_loss / (i + 1)
 
                 # @NOTE: Decrease 900 if you want to plot gradients more often, I find it's too expensive to do so
                 if CONFIG.MONITORING['network_heatmap'] and random.randint(0, 1000) > 900:
@@ -646,15 +663,6 @@ class NnMixer(BaseMixer):
                         self.monitor.weight_map(layer_name, weights, 'Awareness network weights')
                         self.monitor.weight_map(layer_name, weights, 'Awareness network gradients')
 
-                # now that we have run backward in both losses, optimize()
-                # (review: we may need to optimize for each step)
-                self.optimizer.step()
-
-                if self.is_selfaware and self.start_selfaware_training:
-                    self.selfaware_optimizer.step()
-
-                error = running_loss / (i + 1)
-
                 if CONFIG.MONITORING['batch_loss']:
                     #self.monitor.plot_loss(total_loss.item(), self.total_iterations, 'Total Batch Loss')
                     self.monitor.plot_loss(error, self.total_iterations, 'Mean Total Running Loss')
